/*
 * Only use `let` variables to produce the smallest possible bundle.
 * Use `const` only for the exported functions, to avoid mutable
 * exports.
 * Also, instead of using the `Number` function to parse ints from
 * strings, use the shorter unary `+` operator.
 * This isn't pretty, but this utility should not be noticable in your
 * app's bundle.
 */
/**
 * Obfuscate any input string to protect it from being scraped by bots.
 * Use this to obscure email addresses phone numbers etc.
 *
 * @param {String} inputStr The input to scramble. For example an email or
 * a phone number.
 * @param {String} key The key used to scramble the input. This can be any
 * string, that is at least one character long.
 * @returns {String} The obfuscated input. Use `unscramble` to get back the
 * original input.
 *
 * @example
 * ```js
 * import { scramble } from "botex";
 *
 * const key = "abc123xyz";
 * const mail = "my@mail.com";
 * const obfuscatedMail = scramble(mail, key);
 * // -> some gibberish bots hopefully can't read...
 * ```
 */
const scramble = (inputStr, key) => {
    // The codes in the scrabled strings are base36 encoded, so they'll
    // never contain uppercase letters. We use the first 10 uppercase
    // letters to encode the length of each code in base10, simply by
    // replacing each digit from 0-9 with the corresponding letter
    // from A-J
    let base10Codes = "ABCDEFGHIJ";
    // Calculate the length of a code and encode the result as a string
    // using only uppercase letters.
    let stringifyLength = (char) => (char.length + "")
        .split("")
        .map((num) => 
    // Using the unary `+` here is equivalent to `Number(num)`. It's
    // just a bit shorter
    base10Codes[+num])
        .join("");
    let currentLength = "";
    let obfuscated = "";
    // Use map instead of forEach, as it's a bit shorter
    inputStr.split("").map((char, i) => {
        // XOR each letters char code with the char code of the key at
        // the same index.
        // If the input is longer than the key, wrap back to the start
        // of the key, when its length is exceeded
        let keyCode = key.charCodeAt(i % key.length);
        let charCode = char.charCodeAt(0);
        let cypherCode = charCode ^ keyCode;
        // Convert the code to a base 36 string. This way we can shave a few
        // bytes off the length of the encoded string
        let code = cypherCode.toString(36);
        // Only prefix a code with its encoded length, when it is different
        // to the previous length. This way, we don't need to put a separator
        // after every code, to distinguish it from the next; we just store
        // its length, use the length as a separator and, when decoding
        // advance by the length for each code until we hit the next length
        // indicator
        let codeLength = stringifyLength(code);
        if (codeLength !== currentLength) {
            currentLength = codeLength;
            obfuscated += codeLength;
        }
        obfuscated += code;
    });
    return obfuscated;
};

export { scramble as default };
