/**
 * Obfuscate any input string to protect it from being scraped by bots.
 * Use this to obscure email addresses phone numbers etc.
 *
 * @param {String} inputStr The input to scramble. For example an email or
 * a phone number.
 * @param {String} key The key used to scramble the input. This can be any
 * string, that is at least one character long.
 * @returns {String} The obfuscated input. Use `unscramble` to get back the
 * original input.
 *
 * @example
 * ```js
 * import { scramble } from "botex";
 *
 * const key = "abc123xyz";
 * const mail = "my@mail.com";
 * const obfuscatedMail = scramble(mail, key);
 * // -> some gibberish bots hopefully can't read...
 * ```
 */
declare const scramble: (inputStr: string, key: string) => string;
/**
 * Get back the original data, that was obfuscated with the `scramble`
 * function. Use this upon a user interaction to, for example, copy a
 * private phone number to the clipboard or to open an email address in
 * a mail program using a `mailto:` url.
 *
 * @param {String} scrambledStr The input to unscramble. Pass the scrambled
 * email, phone number or other personal data to the `unscramble` function
 * upon a user interaction, to retrieve the original data.
 * @param {String} key The key that was used to scramble the input. This can be
 * any string, that is at least one character long.
 * @returns {String} The original string.
 *
 * @example
 * ```js
 * import { unscramble } from "botex";
 *
 * const key = "abc123xyz";
 * const scrambledMail = "Aasd123Bsdf..."; // Some gibberish...
 * const mailtoLink = document.querySelector("#mail-link");
 *
 * mailtoLink.onclick = () => {
 *   // Get back the original data
 *   const originalMail = unscramble(scrambledMail, key);
 *   location.href = `mailto:${originalMail}`;
 * };
 * ```
 */
declare const unscramble: (scrambledStr: string, key: string) => string;
export { scramble, unscramble };
